; TIcross v1.0 by Dan Whitman
; kyp4@yahoo.com

#include "ti83plus.inc"		; standard TI-83 Plus include file
#include "mirage.inc"		; MirageOS include file

work_ram			= AppBackUpScreen
d_ptr_set			= work_ram				; pointer to start of external puzzle set
d_ptr_vat			= work_ram+2			; VAT pointer for use in set searching
d_num_puzzles		= work_ram+4			; number of puzzles in the current set
d_cursor			= work_ram+5			; current position of the cursor (yx)
d_cursor_rect		= work_ram+7			; current position of cursor rectangle
d_rect_value		= work_ram+9			; value (on | off | flag) to apply to rectangle
d_row_values		= work_ram+10			; values for rows of the current puzzle, 9 bytes * 15 rows
d_col_values		= work_ram+10+135		; values for cols of the current puzzle, 9 bytes * 15 cols
d_thumbnail			= work_ram+10+270		; buffer for display 15 x 15 images
d_thumbnail_buf		= work_ram+10+270+2
d_ba_solved			= TempSwapArea			; first ptr is bit array of puzzles solved
d_arr_ptr_puzzles	= TempSwapArea			; values for puzzle offsets 512 bytes (first address is solved ba)

; NOTE: Due to the size TempSwapArea (323 bytes) the number
; of puzzles allowed in a puzzle set is limited to 160

ScreenWidth			equ 96
ScreenHeight		equ 64
ScreenCenterY		equ ScreenHeight/2
ScreenCenterX		equ ScreenWidth/2

StatsWidth			equ 19
StatsX				equ	ScreenWidth - StatsWidth
KeyGuideY			equ 17

ThumbnailX			equ StatsX + 2
ThumbnailY			equ ScreenHeight - 17

PlayAreaWidth		equ StatsX
PlayAreaCenterX		equ PlayAreaWidth/2

BoardSize			equ 47
BoardDividerLen		equ	15
BoardLocationX		equ	(PlayAreaCenterX)-(BoardSize/2)
BoardLocationY		equ 13
BoardLocationYX		equ	BoardLocationY<<8+BoardLocationX
BoardCenterY		equ BoardLocationY+(BoardSize/2)

CursorSize			equ $04
CursorBlinkDelay	equ 44
CursorDelay			equ	6
CursorRectDelay		equ	CursorDelay

SpaceWidth			equ 2
SpaceHeight			equ 1
CharWidth			equ 4+SpaceWidth
CharHeight			equ 6+SpaceHeight

HorValuesY			equ	3
VerValuesX			equ	(BoardLocationX-8)

PuzzleMessagePad	equ 2
PuzzleMessageHeight	equ 11+(2*PuzzleMessagePad)
PuzzleMessageDelay	equ 20

; key bit equates
tckeyDown			equ	0
tckeyLeft			equ 1
tckeyRight			equ 2
tckeyUp				equ 3
tckeyDel			equ 4
tckey2nd			equ 5
tckeyMode			equ 6
tckeyAlpha			equ 7

;---------= MirageOS Header =---------
	.org $9d93               ; origin
	.db  $BB,$6D             ; compiled AsmPrgm token
	ret	 					 ; header byte 1 -- so TIOS wont run
	.db	1					 ; header byte 2 -- Identifies as MirageOS prog
button:						 ; button - should be a 15x15 graphic
	.db %00000000,%00000000
	.db %01101101,%10110110
	.db %01101101,%10110110
	.db %00000000,%00000000
	.db %01001101,%00100100
	.db %00101100,%10010010
	.db %00000000,%00000000
	.db %01101101,%00110100
	.db %01101100,%10110010
	.db %00000000,%00000000
	.db %01001101,%10110110
	.db %00101101,%10110110
	.db %00000000,%00000000
	.db %01001101,%00110100
	.db %00101100,%10110010

d_str_description:				; description - zero terminated
	.db	"TIcross v1.0        ",0
d_str_description_saved = d_str_description+12

;---------= Initialization =---------
initialization:
	bcall(_GrBufClr)
	bcall(_ForceFullScreen)
	set	fullScrnDraw,(iy+apiFlg4)		; allow draws to use column 95 and row 0
	res	plotLoc,(iy+plotFlags)			; graphic routines write to buffer and display
	set	bufferOnly,(iy+plotFlag3)		; graphic routines write to buffer only
	set	textWrite,(iy+sGrFlags)			; text writing routines write only to buffer
	res	fracDrawLFont,(iy+fontFlags)	; use small font for certain display calls
	res	textInverse,(iy+textFlags)		; use non-inverted text

	ld	a,15							; set the thumbnail buffer up
	ld	bc,2
	ld	hl,d_thumbnail
	bcall(_MemSet)

;---------= Setup =---------
	ld	hl,(progptr)
	ld	(d_ptr_vat),hl
	call find_next_set
	jp	z,check_saved

	; no sets found, print message and exit
	call draw_message_rect
	ld	a,ScreenCenterY-3
	ld	hl,d_str_no_sets
	call centertext
	res	textInverse, (iy+textFlags)
	call update_screen
	call wait_for_2nd
	jp	main_return

check_saved:
	ld	a,(d_puzzle_num)				; check to see if there is a saved game
	or	a
	jr	z,main_menu
	call draw_message_rect				; we'll need this no matter what
	call find_last_used_set				; check that the saved set still exists
	jr	nz,cs_no_set_gone
	ex	de,hl
	bcall(_StrLength)
	add	hl,bc
	inc	hl
	call setup_puzzle_set

	ld	a,ScreenCenterY-3
	ld	hl,d_str_saved_loading
	call centertext
	res	textInverse, (iy+textFlags)
	call update_screen

	call setup_board
	; update the board - this takes a bit
	ld	b,14
cs_board_loop_y:
	ld	c,14
cs_board_loop_x:
	call get_board_element
	call update_board_element
	dec	c
	jp	p,cs_board_loop_x
	dec	b
	jp	p,cs_board_loop_y
	; update the thumbnail
	ld	hl,d_board_set
	ld	de,(ThumbnailY<<8)+ThumbnailX
	call draw_image_thumbnail
	ld	bc,$0000
	ld	(d_cursor),bc
	jp	main_loop
cs_no_set_gone:
	ld	a,ScreenCenterY-6
	ld	hl,d_str_saved_alert
	call centertext
	ld	a,ScreenCenterY+1
	ld	hl,d_str_saved_opts
	call centertext
	res	textInverse, (iy+textFlags)
	call update_screen
cs_key_loop:
	call get_key
	bit	tckeyMode,a
	jp	nz,main_return
	bit	tckey2nd,a
	jr	z,cs_key_loop

;---------= Main Menu =---------
main_menu:
	call find_last_used_set
main_menu_loop:
	call put_menu
	ld	(d_arr_ptr_puzzles),hl		; temporary place for set pointer after name string

	call update_screen
	call keys_debounce
mm_key_loop:
	call get_key
	bit	tckeyMode,a
	jp	nz,normal_exit
	bit	tckeyAlpha,a
	jr	z,mm_check_2nd
	call find_next_set
	jr	main_menu_loop
mm_check_2nd:
	bit	tckey2nd,a
	jr	z,mm_key_loop

	; initialize to puzzle 1
	ld	a,1
	ld	(d_puzzle_num),a

	; setup the puzzle set
	ld	hl,(d_arr_ptr_puzzles)		; retreive puzzle pointer from temporary memory
	call setup_puzzle_set

;---------= Puzzle Menu =---------
NumSpace	equ 4
ArrowSpace	equ 2
TitleSpace	equ 3
PicX1		equ ScreenCenterX-8
PicX2		equ ScreenCenterX+8
PicY1		equ ScreenCenterY-8
PicY2		equ ScreenCenterY+8

puzzle_menu:
	ld	hl,$0101						; need to set the cursor such that up and left arrows will work
	ld	(d_cursor),hl
	ld	hl,$0A15
	ld	de,$4728
	call clear_rect
	ld	hl,d_img_arrow_up
	ld	de,((ScreenCenterY-2-5-ArrowSpace)<<8)+PicX1-NumSpace-7
	bcall(_DisplayImage)
	ld	hl,d_img_arrow_down
	ld	de,((ScreenCenterY+3+ArrowSpace)<<8)+PicX1-NumSpace-7
	bcall(_DisplayImage)

pm_key_loop:
	ld	hl,PicY2+TitleSpace
	ld	de,(ScreenWidth<<8)+PicY2+TitleSpace+5
	call clear_rect
	
	call get_ba_solved							; check if the puzzle is solved
	and	(hl)
	jr	nz,pm_solved							; unsolved so use ?
	ld	hl,d_img_unsolved
	jr	pm_put_puzzle
puzzle_menu_fresh:
	call put_menu								; this is for coming to puzzle menu directly from engine
	jr	puzzle_menu
pm_solved:
	call get_puzzle_ptr
	push hl
	ld	bc,30
	add	hl,bc
	ld	a,PicY2+TitleSpace						; solved so use actual pic and show title
	call centertext
	pop	hl
	call get_puzzle_ptr
pm_put_puzzle:
	push hl
	ld	hl,((ScreenCenterY-3)<<8)+PicX1-NumSpace-9
	call put_puzzle_number
	ld	de,((PicY1+1)<<8)+PicX1+1
	pop	hl
	call draw_image_thumbnail
	ld	hl,(PicY1<<8)+PicX1
 	ld	de,(PicY2<<8)+PicX2
	bcall(_DrawRectBorder)
	call update_screen

	call get_key
	bit	tckeyMode,a
	jp	nz,main_menu
	bit	tckey2nd,a
	jr	nz,main_engine
	bit	tckeyDel,a
	jr	z,pm_check_alpha			; reset the set, set the solved bit array to all 0
 	ld	a,$FD						; check that clear is held
	call directin
	and	%01000000
	jr	nz,pm_key_loop
	ld	a,(d_num_puzzles)
	dec	a							; puzzle 1 is bit array element 0
	srl	a							; divide by 8 to get number of bytes of ba
	srl	a
	srl	a
	inc	a							; whatever byte the last puzzle is in + 1 is number of bytes
	cp	1							; if 1 cannot use MemSet
	push af
	ld	hl,(d_ba_solved)
	ld	c,a
	ld	b,0
	xor	a
	ld	(hl),a
	pop	af
	jr	z,pm_key_loop
	xor	a
	bcall(_MemSet)
	jp	pm_key_loop
pm_check_alpha:
	bit	tckeyAlpha,a
	jr	z,pm_check_up
	ld	a,(d_num_puzzles)
	ld	b,a
	xor	a
pm_find_first_loop:
	ld	hl,(d_ba_solved)
	push bc
	ld	b,a
	push af
	call get_bit_array
	and	(hl)
	pop	de
	ld	a,d
	pop	bc
	jr	nz,pm_ff_solved
	inc	a
	ld	(d_puzzle_num),a
	ld	b,1
pm_ff_solved:
	inc	a
	djnz pm_find_first_loop
	jp	pm_key_loop
pm_check_up:
	ld	hl,d_puzzle_num
	bit	tckeyUp,a
	jr	z,pm_check_down
	inc	(hl)
	ld	a,(d_num_puzzles)
	inc	a
	cp	(hl)
	jp	nz,pm_key_loop
	ld	a,1
	ld	(hl),a
	jp	pm_key_loop
pm_check_down:
	bit	tckeyDown,a
	jp	z,pm_key_loop
	dec	(hl)
	jp	nz,pm_key_loop
	ld	a,(d_num_puzzles)
	ld	(hl),a
	jp	pm_key_loop

;---------= Main Engine =---------
main_engine:
	; clear the board memory
	ld	hl,d_board_set
	ld	bc,60
	bcall(_MemClear)
	call setup_board
main_loop:
	call invert_cursor
	call update_screen
main_loop_no_invert:
	push bc
	ld b,CursorBlinkDelay
	ei
cursor_delay_loop:
	halt
	call query_keys
	jr	nz,key_pressed
	djnz cursor_delay_loop
	pop bc
	jr	main_loop
key_pressed:
	pop	bc

	bit	tckeyMode,a
	jp	nz,puzzle_menu_fresh
	bit	tckeyDel,a
	jr	nz,save_exit

	push af
 	and	(1<<tckeyAlpha) | (1<<tckey2nd)
 	jr	z,no_rect
 	pop	af
 	call cursor_rect_handler
 	; need to see if puzzle is solved
 	push bc
	call get_puzzle_ptr
	ld	de,d_board_set
	ld	b,30
	call compstrsn
	pop	bc
 	jr	nz,main_loop
	call puzzle_solved
	jp	puzzle_menu_fresh					; go back to puzzle menu
no_rect:
	pop	af
	call apply_keys
move_cursor:
	push bc
	ld	bc,(d_cursor)
	call clear_cursor
	pop bc
	call draw_cursor
	ld	(d_cursor),bc
	call put_values_hor
	call put_values_ver
	call update_screen
	ld	a,CursorDelay
	call delay
	jr	main_loop_no_invert
out_of_bounds:
	ld	bc,(d_cursor)
	jr	main_loop
normal_exit:
	xor	a								; mark as no saved game
	ld	(d_puzzle_num),a
	ld	hl,d_str_description_saved		; remove saved from label
	ld	bc,StrSavedLen
	xor									; set all to zero, they may get executed but its ok they're NOPs
	bcall(_MemSet)
	jr	main_return
save_exit:
	ld	hl,d_str_saved					; put saved in the game title
	ld	de,d_str_description_saved
	ld	bc,StrSavedLen
	ldir
main_return:
	call keys_debounce
    set	apdAble,(iy+apdFlags)			; re-enable APD
	di
	ret

;-----------------------------------------------------------------
; Draws main menu
; (no input or output registers)
;-----------------------------------------------------------------
put_menu:
	bcall(_GrBufClr)
	set	fracDrawLFont,(iy+fontFlags)
	xor	a
	ld	hl,d_str_title
	call centertext
	res	fracDrawLFont,(iy+fontFlags)
	ld	hl,$0008
	ld	de,$5F08
	ld	a,1
	call fastline
	ld	de,$0900
	ld	hl,d_str_menu_screen
	call setvputs
	ld	de,$150A
	call setvputs
	ld	de,$1C0A
	call setvputs
	ld	de,$230A
	call setvputs
	ld	de,$3800
	call setvputs
	ld	hl,(d_ptr_set)
	bcall(_VPutS)
	ret

;-----------------------------------------------------------------
; Sets up the puzzle set
; Input:
;	hl = address of the puzzle set after the name string
;-----------------------------------------------------------------
setup_puzzle_set:
	ld	de,d_set_id					; copy the 3 byte set identifier
	ld	bc,3
	ldir
 	ld	a,(hl)
 	ld	(d_num_puzzles),a
 	inc	hl
 	ld	(d_ba_solved),hl
 	ld	b,a
 	push bc
 	dec	b							; how many bytes is the bit array?
	call get_bit_array
	inc	hl
	ld	de,d_arr_ptr_puzzles+2		; initialize to puzzle 1 pointer
	pop	bc
	ex	de,hl
puzzle_setup_loop:
	ld	(hl),d						; store address to current arr element
	inc	hl
	ld	(hl),e
	dec	b
	ret	z
	inc	hl
	push hl							; skip the 30 bytes of puzzle data
	ld	hl,30
	add	hl,de
	push bc
	bcall(_StrLength)
	add	hl,bc
	pop	bc
	inc	hl
	ex	de,hl
	pop	hl
	jr	puzzle_setup_loop
	ret


;-----------------------------------------------------------------
; Draws the board stuff as well as builds the values
; (no input or output registers)
;-----------------------------------------------------------------
setup_board:
	bcall(_GrBufClr)

	; draw main board rect
	ld	hl,BoardLocationYX
	ld	de,BoardLocationYX+(BoardSize<<8 + BoardSize)
	bcall(_DrawRectBorder)

	; draw dividers
	ld	de,d_divider_points
	call put_points

	; draw stats area
	ld	hl,StatsX
	ld	de,((ScreenHeight-1)<<8)+ScreenWidth-1
	bcall(_DrawRectBorder)

	; annoyingly enough the last "z" must be done like this because it goes off the screen otherwise
	set textInverse,(iy+textFlags)
	ld	de,StatsX+13
	ld	(pencol),de
	ld	a,$7A
	bcall(_VPutMap)

	; key guide
	ld	de,StatsX+1
	ld	hl,str_key_guide
	call setvputs
	ld	de,(KeyGuideY<<8)+StatsX+1
	call setvputs
	ld	de,((KeyGuideY+14)<<8)+StatsX+1
	call setvputs
	res	textInverse,(iy+textFlags)
	ld	de,((KeyGuideY+7)<<8)+StatsX+2
	call setvputs
	ld	de,((KeyGuideY+21)<<8)+StatsX+6
	call setvputs
	; puzzle number
	ld	hl,((8)<<8)+ThumbnailX+2
 	call put_puzzle_number
	; line dividing thumbnail
	ld	hl,((ThumbnailX-1)<<8)+ThumbnailY-2
	ld	de,((ThumbnailX+15)<<8)+ThumbnailY-2
	ld	a,1
	call fastline

	; draw lines seperating sections
	ld	hl,((ThumbnailX-1)<<8)+6
	ld	de,((ThumbnailX+15)<<8)+6
	ld	a,1
	call fastline
	ld	hl,((ThumbnailX-1)<<8)+KeyGuideY+6
	ld	de,((ThumbnailX+15)<<8)+KeyGuideY+6
	ld	a,1
	call fastline
	ld	hl,((ThumbnailX-1)<<8)+KeyGuideY+20
	ld	de,((ThumbnailX+15)<<8)+KeyGuideY+20
	ld	a,1
	call fastline

	ld	bc,$0000
	ld	(d_cursor),bc
	push bc
	call build_all_values
	pop	bc
	call put_values_hor
	call put_values_ver
	ret

str_key_guide:
	.db "Puz"
	.db 0
	.db "Quit    "
	.db 0
	.db "Save "
	.db 0
	.db "MODE"
	.db 0
	.db "DEL"
	.db 0

;-----------------------------------------------------------------
; Draws the black centered rectangle used for messages
; (no input or output registers)
;-----------------------------------------------------------------
draw_message_rect:
	ld	hl,(ScreenCenterY-6-PuzzleMessagePad)<<8
	ld	de,((ScreenCenterY+7+PuzzleMessagePad)<<8)+ScreenWidth-1
	bcall(_FillRect)
	set	textInverse, (iy+textFlags)
	ret

;-----------------------------------------------------------------
; Find the next puzzle set and if there isn't one wraps around to
; the first one
; Output:
;	z = another puzzle set found
;	hl = address of puzzle set
;-----------------------------------------------------------------
find_next_set:
	ld	hl,(d_ptr_vat)
	ld	ix,d_str_detect
	call iDetect
	jr z,fns_found
	ld	hl,(progptr)				; loop back around to first set
	call iDetect
fns_found:
	ld	(d_ptr_vat),de
	ld	(d_ptr_set),hl
	ret

;-----------------------------------------------------------------
; Searches for the last set that was plyed based on set ID
; Output:
;	z = last used set was found
;	hl = address of last used set
;-----------------------------------------------------------------
find_last_used_set:
	ld	hl,(progptr)
	ld	ix,d_str_detect
flus_loop:
	call iDetect
	ret	nz				; no more sets, we're done and set not found
	push de
	push hl
	bcall(_StrLength)		; skip the set name
	add	hl,bc
	inc	hl
	ld	de,d_set_id		; look for the set identifier
	ld	b,3
	call compstrsn
	pop	de
	pop	hl
	jr	nz,flus_loop
    ld	(d_ptr_vat),hl
	ld	(d_ptr_set),de
	ret

;-----------------------------------------------------------------
; Gets element from a bit array at specified address
; Input:
;	hl = start addr
;	b = bit number
; Output:
;	hl = byte addr
;	a = bitmask
;-----------------------------------------------------------------
get_bit_array:
	push bc
	srl	b							; divide by 8 to get byte offset
	srl	b
	srl	b

	ld	d,0							; add offset to hl
	ld	e,b
	add	hl,de
	pop	af

	and	%00000111					; get bit number in the byte (mod 8)
	ld	b,a
	ld	a,$80
	inc	b							; need to check if b is zero, this is annoying
	dec	b
	jr	z,gbae_no_shift
gbae_shift_loop:
	srl	a
	djnz gbae_shift_loop
gbae_no_shift:
	ret

;-----------------------------------------------------------------
; Draws a 15x15 thumbnail
; Input:
;	hl = address of thumbnail to be drawn (2 bytes/row)
;	de = coordinates to draw the thumbnail (yx)
;-----------------------------------------------------------------
draw_image_thumbnail:
	push de
	ld	de,d_thumbnail_buf
	ld	bc,30
	ldir
	pop	de
	push hl
	ld	hl,d_thumbnail
	bcall(_DisplayImage)
	pop	hl
	ret

;-----------------------------------------------------------------
; Gets address of current puzzle number
; Output:
;	hl = address of current puzzle (starting at puzzle data)
;-----------------------------------------------------------------
get_puzzle_ptr:
	push bc
	push de
	ld	a,(d_puzzle_num)
	ld	c,a
	ld	b,0
	ld	hl,d_arr_ptr_puzzles
	sla	c							; 2 bytes / element
	rl  b							; 16-bit shift
	add	hl,bc
	ld	d,(hl)
	inc	hl
	ld	e,(hl)
	ex	de,hl
	pop	de
	pop	bc
	ret

;-----------------------------------------------------------------
; Waits for user to press a one of the used keys
; Output:
;	a = key(s) pressed (see query_keys for bit values)
;-----------------------------------------------------------------
get_key:
	ei
gk_loop:
	halt
	call query_keys
	jr	z,gk_loop
	push af
	call keys_debounce
	pop	af
	ret
	
;-----------------------------------------------------------------
; Halts until 2nd key is pressed by the user
; (no input or output registers)
;-----------------------------------------------------------------
wait_for_2nd:
	call get_key
	bit	tckey2nd,a
	jr	z,wait_for_2nd
	ret

;-----------------------------------------------------------------
; Debounces any keys pressed, which is to say if any keys are
; currently pressed, halts until no keys are pressed
; (no input or output registers)
;-----------------------------------------------------------------
keys_debounce:
kd_loop:
	call query_keys
	jr nz,kd_loop
	call query_keys
	jr nz,kd_loop
	call query_keys
	jr nz,kd_loop
	ret

;-----------------------------------------------------------------
; Updates the screen from graph buffer while preserving bc
; (no input or output registers)
;-----------------------------------------------------------------
update_screen:
	push bc
	bcall(_GrBufCpy)
	pop	bc
	ret

;-----------------------------------------------------------------
; Draws a set of points to graph buffer
; Input:
;	de = address of points to be drawn, first byte = number of pts
;		 and subsequent points are (xy) with 2 bytes/pt
;-----------------------------------------------------------------
put_points:
	ld	a,(de)
	ld	b,a
pp_loop:
	inc	de
	ld	a,(de)
	ld	h,a
	inc	de
	ld	a,(de)
	ld	l,a
	push de
	push bc
	call pixelonhl
	pop bc
	pop	de
	djnz pp_loop
	ret

;-----------------------------------------------------------------
; Given keys from query_keys, applies key presses to coordinates in bc
; Input:
;	a = key bits from query_keys
;	bc = current coordinates (yx)
; Output:
;	bc = new coordinates (yx)
;-----------------------------------------------------------------
apply_keys:
	bit	tckeyLeft,a
	jr	z,ak_check_right
	dec	c
	jr	ak_check_up
ak_check_right:
	bit	tckeyRight,a
	jr	z,ak_check_up
	inc	c
ak_check_up:
	bit	tckeyUp,a
	jr	z,ak_check_down
	dec	b
	jr	ak_check_keys
ak_check_down:
	bit	tckeyDown,a
	jr	z,ak_check_keys
	inc	b
ak_check_keys:
	and	$0F
	ret

;-----------------------------------------------------------------
; Given board element coordinates finds the offset in memory
; Input:
;	bc = current coordinates (yx)
; Output:
;	a = bitmask of board element
;	de = offset of board element
;-----------------------------------------------------------------
get_board_offset:
	sla	b							; 2 bytes / row
	push hl
	ld	l,b
	ld	h,0
	ld	b,c
	call get_bit_array
	ex	de,hl
	pop	hl
	ret

;-----------------------------------------------------------------
; Gets board element
; Input:
;	bc = element coordinates (yx)
; Output:
;	a = board element (0 = nothing, 1 = flag, 2 = set)
;-----------------------------------------------------------------
get_board_element:
	push bc
	push de
 	call get_board_offset
 	ld	hl,d_board_set
 	push af							; store the bitmask for later
 	add	hl,de
 	and	(hl)
 	jr	z,gbe_no_set
 	ld	a,2							; set, so set bit 1
gbe_no_set:
	ld	hl,d_board_flag
	add	hl,de
	ld	d,a							; done with de now, so store the set value for later
	pop	af							; retreive the bitmask
	and	(hl)
	jr	z,gbe_no_flag
	inc	d							; flag set, set set bit 0
gbe_no_flag:
	ld	a,d
	pop	de
	pop bc
	ret

;-----------------------------------------------------------------
; Sets the board element and also then updates the screen
; (by simply continuing execution on through update_board_element)
; Input:
;	a = board element (0 = nothing, 1 = flag, 2 = set)
;	bc = element coordinates (yx)
;-----------------------------------------------------------------
set_board_element:
	push bc
	push af

	push af
	call get_board_offset
 	ld	hl,d_board_set
 	pop	bc
 	ld	c,2							; bitmask for the set value, this time storing set
 	call sbe_set_one
 	ld	hl,d_board_flag
 	ld	c,1							; bitmask for the set value, this time storing flag
 	call sbe_set_one

	; now update the graph buffer
	pop	af
	pop	bc

;-----------------------------------------------------------------
; Updates one element to the screen
; Input:
;	a = board element (0 = nothing, 1 = flag, 2 = set)
;	bc = element coordinates (yx)
;-----------------------------------------------------------------
update_board_element:
	push bc
	push af
	call get_element_rect
	pop	af
	bit	1,a
	jr	z,ube_not_set
	bcall(_DrawRectBorder)
	jr	ube_return
ube_not_set:
	push hl
	bcall(_EraseRectBorder)
	pop	hl
	bit	0,a
	jr	z,ube_return
	ld	d,h
	ld	h,l
	ld	l,d
	push hl
	call pixelonhl
	pop	hl
	ld	de,$0101
	add	hl,de
	call pixelonhl
ube_return:
	pop	bc
	ret

;-----------------------------------------------------------------
; Sub-routine of set_board_element, don't use independently
;-----------------------------------------------------------------
sbe_set_one:
	push af
	push de
	add	hl,de
	push af
	pop	de
	ld	a,b							; test whichever bit we're dealing with
	and	c
	ld	a,d
	jr	nz,sbeso_one
	cpl								; reset memory bit
	and	(hl)
	jr	sbeso_return
sbeso_one:
	or	(hl)						; set memory bit
sbeso_return:
	ld	(hl),a
	pop	de
	pop	af
	ret

;-----------------------------------------------------------------
; Sub-routine of cursor_rect_handler, don't use independently
;-----------------------------------------------------------------
crh_test_keys:
	call query_keys
crh_d_bit:
	bit	0,a						; the bit num will be cleverly set in the crh routine
	ret

;-----------------------------------------------------------------
; Handles all aspects of setting/flag rectangle
; Input:
;	a = key(s) pressed (see query_keys for bit values)
;-----------------------------------------------------------------
cursor_rect_handler:
	push af
	push af
	call draw_cursor
	call update_screen
	pop	af
	bit	tckey2nd,a
	jr	z,crh_alpha_pressed
	ld	a,$47+(8*tckey2nd)		; set the check bit num below to 2nd key: opcode is 2 bytes: CB 40+8*b+rb
	ld	d,~1					; invert the set bit and reset the set bit
	jr	crh_begin
crh_alpha_pressed:
	ld	a,$47+(8*tckeyAlpha)	; set the check bit num below to alpha key: opcode is 2 bytes: CB 40+8*b+rb
	ld	d,~2					; invert the set bit and reset the flagged bit
crh_begin:
	ld	(crh_d_bit+1),a
	call get_board_element
	cpl
	and	d
	ld	(d_cursor),bc
	ld	(d_cursor_rect),bc
	ld	(d_rect_value),a
crh_key_loop:
	call crh_test_keys			; because of slight glitches in the key driver, must query 3 times
    jr	nz,crh_key_still_down
    call crh_test_keys
    jr	nz,crh_key_still_down
    call crh_test_keys
    jr	nz,crh_key_still_down
	jr	crh_key_release
crh_key_still_down:
	and	$0F
	jr	z,crh_key_loop
	call apply_keys

	push bc
	call put_values_hor
	call put_values_ver
 	call get_cursor_rect_rect
 	bcall(_EraseRectBorder)
 	pop	bc
 	ld	(d_cursor),bc
 	call get_cursor_rect_rect
 	bcall(_DrawRectBorder)
 	call update_screen

 	ld	a,CursorRectDelay
 	call delay

	ld	bc,(d_cursor)
	jr	crh_key_loop
crh_key_release:
	call rectify				; now we must apply the value to the whole rectangle

    push hl
    pop	bc
crh_apply_loop_y:
crh_apply_loop_x:
	push hl
	push de
	ld	a,(d_rect_value)
	call set_board_element
	pop	de
	pop	hl
	inc	c
	ld	a,e
	cp	c
	jp	p,crh_apply_loop_x
	ld	c,l
	inc	b
	ld	a,d
	cp	b
	jp	p,crh_apply_loop_y

	call get_cursor_rect_rect
	bcall(_EraseRectBorder)
	ld	hl,d_board_set						; update the thumbnail
	ld	de,(ThumbnailY<<8)+ThumbnailX
	call draw_image_thumbnail
	ld	bc,(d_cursor)
	call put_values_hor
	call put_values_ver
	pop af
	ret

;-----------------------------------------------------------------
; Uses direct input to get keys currently pressed (limited to
; keys in which we're interested for this program)
; Output:
;	a = keys currently pressed (set bit indicates pressed key):
;		Bit 0: Down
;		Bit 1: Left
;		Bit 2: Right
;		Bit 3: Up
;		Bit 4: Del
;		Bit 5: 2nd
;		Bit 6: Mode
;		Bit 7: Alpha
;-----------------------------------------------------------------
query_keys:
	push bc
	ld	a,%11111110		; get arrow keys
	call directin
	or	%11110000		; filter out wanted keys
	ld	b,a
	ld	a,%10111111		; get 2nd and Mode
	call directin
	or	%10011111		; filter out wanted keys
	ld	c,a
	ld	a,%11011111		; get alpha
	call directin
	or	%01111111		; filter out wanted keys
	and	b
	and	c
	cpl
	ld	b,a
	ld	a,%10111111		; get del
	call directin
	or	%01111111		; filter out wanted keys
	cpl
	rra \ rra \ rra
	or	b
	; check the bounds and disregard if exceeded
	ld	d,a
	and	$0F
	jr	z,qk_return		; this isn't really necessary but saves a bit of time if no arrows are pressed
	ld	a,(d_cursor)
	or	a
	jr	nz,qk_check_right
	res	1,d
	jr	qk_check_up
qk_check_right:
	cp	$0E
	jr	nz,qk_check_up
	res	2,d
qk_check_up:
	ld	a,(d_cursor + 1)
	or	a
	jr	nz,qk_check_down
	res	3,d
	jr	qk_return
qk_check_down:
	cp	$0E
	jr	nz,qk_return
	res	0,d
qk_return:
	ld	a,d
	or	a
	pop	bc
	ret

;-----------------------------------------------------------------
; Delay
; Input:
;	a = Delay time
;		(not really a specified time just, the higher, the more delay)
;-----------------------------------------------------------------
delay:
	push bc
	ld	b,a
delay_loop:
	ei
	halt
	djnz delay_loop
	pop bc
	ret


;-----------------------------------------------------------------
; Gets screen coordinates for a a board element
; Input:
;	bc = element coordinates (yx)
; Output:
;	hl = upper left screen coordinates (yx)
;	de = lower right screen coordinates (yx)
;-----------------------------------------------------------------
get_element_rect:
	push bc
	ld	h,b
	ld	l,3
	call multhl
	ld	a,l
	ld	h,c
	ld	l,3
	call multhl
	ld	h,a
	ld	de,BoardLocationYX+$0202
	add	hl,de
	push hl
	pop	de
	inc	d
	inc	e
	pop bc
	ret


;-----------------------------------------------------------------
; For the cursor and rect board coordinates determines which to
; use for upper and lower rectangle coordinates
; Output:
;	hl = upper left board coordinates (yx)
;	de = lower right board coordinates (yx)
;-----------------------------------------------------------------
rectify:
	ld	hl,(d_cursor_rect)
	ld	de,(d_cursor)
	ld	a,d
	cp	h
	jp	p,r_no_swap1
	ld	d,h
	ld	h,a
r_no_swap1:
	ld	a,e
	cp	l
	jp	p,r_no_swap2
	ld	e,l
	ld	l,a
r_no_swap2:
	ret

;-----------------------------------------------------------------
; For the cursor and rect board coordinates gets the bounding
; screen rectangle
; Output:
;	hl = upper left screen coordinates (yx)
;	de = lower right screen coordinates (yx)
;-----------------------------------------------------------------
get_cursor_rect_rect:
	call rectify
	push de
	push hl
	pop	bc
	call get_cursor_rect
	pop	bc
	push hl
	call get_cursor_rect
	pop	hl
	ret

;-----------------------------------------------------------------
; For given board element determines cursor rectangle bounds
; Input:
;	bc = cursor board coordinates
; Output:
;	hl = upper left board coordinates (yx)
;	de = lower right board coordinates (yx)
;-----------------------------------------------------------------
get_cursor_rect:
	ld	h,b
	ld	l,3
	call multhl
	ld	a,l
	ld	h,c
	ld	l,3
	call multhl
	ld	h,a
	ld	de,BoardLocationYX+$0101
	add	hl,de
	push hl
	ld	de,CursorSize-1<<8+CursorSize-1
	add	hl,de
	ex	de,hl
	pop	hl
	ret

;-----------------------------------------------------------------
; Inverts the cursor rectangle in graph buffer
; Input:
;	bc = cursor board coordinates
;-----------------------------------------------------------------
invert_cursor:
	push bc
	call get_cursor_rect
	ld	a,h
	ld	h,l
	ld	l,a
	call pixeltesthl
	pop	bc
	jr	nz,ic_clear
	call draw_cursor
	ret
ic_clear:
	call clear_cursor
	ret

;-----------------------------------------------------------------
; Draws the cursor rectangle in the graph buffer
; Input:
;	bc = cursor board coordinates
;-----------------------------------------------------------------
draw_cursor:
	push bc
	call get_cursor_rect
	bcall(_DrawRectBorder)
	pop	bc
	ret

;-----------------------------------------------------------------
; Clears the cursor rectangle in the graph buffer
; Input:
;	bc = cursor board coordinates
;-----------------------------------------------------------------
clear_cursor:
	push bc
	call get_cursor_rect
	bcall(_EraseRectBorder)
	pop	bc
	ret


;-----------------------------------------------------------------
; Calculates and applies memory offset for a value row/col
; Input:
;	b = value row/col
;	de = pointer to start of values
; Output:
;	de = pointer to start of values for row/col
;-----------------------------------------------------------------
apply_values_offset:
	push bc
	push de
	ld	e,b
	ld	h,9
	call multhe
	pop	de
	add	hl,de
	pop	bc
	ret

;-----------------------------------------------------------------
; Gets next bit when iterating through bit array
; NOTE: d_gnc_bitmask must be initialized prior to use
; Input:
;	hl = current byte
; Output:
;	z = bit zero (not set)
;-----------------------------------------------------------------
d_gnc_bitmask		equ d_cursor_rect
get_next_col:
	ld	a,(d_gnc_bitmask)
	and	(hl)
	push af
	ld	a,(d_gnc_bitmask)
	srl	a
    jr	nz,gnc_same_byte
	ld	a,$80						; time to move to the next byte
	inc	hl
gnc_same_byte:
	ld	(d_gnc_bitmask),a
	pop	af
	ret
	
;-----------------------------------------------------------------
; Gets next bit when iterating through rows with same column
; NOTE: d_gnr_bitmask must be initialized prior to use to constant
; Input:
;	hl = current byte
;	d_gnr_bitmask = column number
; Output:
;	z = bit zero (not set)
;-----------------------------------------------------------------
get_next_row:
	; since there isn't the extra zero at the end check if on last one and if so return zero
	ld	a,b
	or	a
	cp	1
	ret	z
	; now do a real bit
gnr_bitmask:
d_gnr_bitmask		equ gnr_bitmask+1
	ld	a,0							; this will be set to whatever col bitmask by build_all_values
	and	(hl)
	push af
	inc	hl							; move to next row
	inc	hl
	pop	af
	ret

;-----------------------------------------------------------------
; Builds all values for a puzzle
;-----------------------------------------------------------------
build_all_values:
	; build values for the rows first (hor)
	ld	hl,get_next_col
	ld	(d_bv_to_call),hl
	
	ld	b,0
bav_row_loop:
	ld	de,d_row_values				; setup location to put values
	call apply_values_offset
	ex	de,hl
	ld	a,$80
	ld	(d_gnc_bitmask),a

	push bc
	call get_puzzle_ptr				; setup pointer to appropriate row
	ld	c,b
	ld	b,0
	sla	c							; 2 bytes/row
	add	hl,bc
	call build_values
	pop	bc
	inc	b
	ld	a,b
	cp	15
	jr	nz,bav_row_loop

	; now build values for the collumns (ver)
	ld	hl,get_next_row
	ld	(d_bv_to_call),hl

	ld	b,0
bav_col_loop:
	ld	de,d_col_values				; setup location to put values
	call apply_values_offset

	push bc
	push hl
	call get_puzzle_ptr				; setup pointer to appropriate col
	ld	a,b
	cp	8
	jp	m,bav_first_byte
	inc	hl
bav_first_byte:
	ld	c,a
	call get_board_offset

	ld	(d_gnr_bitmask),a
	pop	de
	pop	bc
	push bc
	call build_values
	pop	bc
	inc	b
	ld	a,b
	cp	15
	jr	nz,bav_col_loop
	ret

build_values:
	di								; so we can use af' without problems
	ld	bc,$1000					; initialize counter (b) and stream count (c)
	xor	a							; initialize last bit (to zero)
	ex	af,af'
bv_loop:
d_bv_to_call		equ bv_loop+1
	call 0							; this will be set in build_all_values
	push af
	jr	nz,bv_one
	ex	af,af'						; zero so we need to check the last bit
	jr	z,bv_continue				; both zero so do nothing
	ld	a,c
	ld	(de),a
	ld	c,0
	inc	de
	jr	bv_continue
bv_one:
	inc	c
bv_continue:
	pop	af
	ex	af,af'						; set this to last
	djnz bv_loop
	xor	a
	ld	(de),a						; now close it with zero
	ret

;-----------------------------------------------------------------
; Writes a decimal number to the graph buffer at specified location
; Input:
;	a = number to write
;	hl = screen coordinates to write (yx)
;-----------------------------------------------------------------
put_number_loc:
	ld	(pencol),hl

;-----------------------------------------------------------------
; Writes a decimal number to the graph buffer at current cursor position
; Input:
;	a = number to write
;-----------------------------------------------------------------
put_number:
	push hl
	push af
	ld	l,a
	ld	h,0
	bcall(_SetXXXXOp2)
	bcall(_Op1ExOp2)
	ld	a,3
	bcall(_DispOp1a)
	pop af
	pop	hl
	ret
	
;-----------------------------------------------------------------
; Puts puzzle number padded with zeros
;	hl = location on the screen (yx)
;-----------------------------------------------------------------
put_puzzle_number:
	ld	a,(d_puzzle_num)
	or	a							; if bit 7 is set then 3 digits
	jp	m,ppn_put
	cp	100
	jp	p,ppn_put
	call ppn_put_zero
	cp	10
	call m,ppn_put_zero
ppn_put:
	call put_number_loc
	ret
ppn_put_zero:
	push af
	xor	a
	call put_number_loc
	pop	af
	ld	bc,4
	add	hl,bc
	ret

;-----------------------------------------------------------------
; Clears a rectangle in the graph buffer only
; Input:
;	hl = upper left coordinates (xy)
;	de = lower right coordinates (xy)
;-----------------------------------------------------------------
clear_rect:
	push bc
	ld	a,e
	inc	a
	ld	e,l
cr_loop:
	push af
	xor	a
	push hl
	push de
	call fastline
	pop	de
	pop	hl
	pop	af
	inc	l
	inc	e
	cp	l
	jr	nz,cr_loop
	pop	bc
	ret

;-----------------------------------------------------------------
; Displays row values for a specified row
; Input:
;	b = row values to display
;-----------------------------------------------------------------
HorClearRectU		equ (HorValuesY+1)+(PlayAreaCenterX-((8*CharWidth-1-SpaceWidth)/2)<<8)
HorClearRectL		equ (HorValuesY+5)+(PlayAreaCenterX+((8*CharWidth-1-SpaceWidth)/2)<<8)

put_values_hor:
	push bc
	; first clear the presently displayed values
 	ld	hl,HorClearRectU
 	ld	de,HorClearRectL
	call clear_rect

	ld	de,d_row_values
	call apply_values_offset
	push hl

	; now we need first to calculate where to put the values (centered on the board)
	ld	b,0
uhv_width_loop:
	ld	a,(hl)
	or	a
	jr	z,uhv_loop_done
	cp	10
	jp	m,uhv_one_digit
	ld	a,8+SpaceWidth				; a double digit number is twice as wide
	jr	uhv_loop_add
uhv_one_digit:
	ld	a,4+SpaceWidth
uhv_loop_add:
	add	a,b
	ld	b,a
	inc	hl
	jr	uhv_width_loop
uhv_loop_done:
	ld	a,b
	or	a
	jr	nz,uhv_width_nonzero
	ld	a,4+SpaceWidth
uhv_width_nonzero:
	sub	SpaceWidth+1				; lop off the trailing space
	; at this point the width of the string is in a
	sra	a
	ld	b,a
	ld	a,PlayAreaCenterX
	sub	b
	ld	h,HorValuesY
	ld	l,a
	ld	(pencol),hl

	; now the cursor is set to start writing the numbers
	pop	hl
	push hl
uhv_put_loop:
	ld	a,(hl)
	or	a
	jr	z,uhv_put_loop_done
	call put_number
	inc	hl
	jr	uhv_put_loop
uhv_put_loop_done:
	pop	de
	call cphlde
	jr nz,uhv_return
	xor	a
	call put_number
uhv_return:
	pop	bc
	ret
	
;-----------------------------------------------------------------
; Displays col values for a specified col
; Input:
;	c = col values to display
;-----------------------------------------------------------------
VerClearRectU		equ (BoardCenterY-((8*CharHeight-1-SpaceHeight)/2))+((VerValuesX-3)<<8)
VerClearRectL		equ (BoardCenterY+((8*CharHeight-1-SpaceHeight)/2)-1)+((VerValuesX+3)<<8)

put_values_ver:
	push bc
	; first clear the presently displayed values
	ld	hl,VerClearRectU
 	ld	de,VerClearRectL
	call clear_rect

	ld	b,c
	ld	de,d_col_values
	call apply_values_offset
	push hl

	; now we need first to calculate where to put the values (centered on the board)
	ld	e,0
uvv_height_loop:
	ld	a,(hl)
	or	a
	jr	z,uvv_loop_done
	inc	e
	inc	hl
	jr	uvv_height_loop
uvv_loop_done:
	ld	a,e
	or	a
	jr	nz,uvv_height_nonzero
	inc	e
uvv_height_nonzero:
	ld	h,6+SpaceHeight
	call multhe
	dec	l						; lop off trailing space
	dec	l
	; at this point the height of the string is in l
	sra	l
	ld	a,BoardLocationY+(BoardSize/2)-1
	sub	l
 	ld	(penrow),a

	; now the cursor Y is set so start writing the numbers
	pop	hl
	push hl
uvv_put_loop:
	ld	a,VerValuesX-(3/2)		; set pencol X for one digit number
	ld	(pencol),a
	ld	a,(hl)
	or	a
	jr	z,uvv_put_loop_done
	cp	10
	jp	m,uvv_one_digit
	push af
	ld	a,VerValuesX-(7/2)		; set pencol X for two digit number
	ld	(pencol),a
	pop	af
uvv_one_digit:
	call put_number
	inc	hl
	ld	a,(penrow)
	add	a,6+SpaceHeight			; increment pencol Y by one char (plus space)
	ld	(penrow),a
	jr	uvv_put_loop
uvv_put_loop_done:
	pop	de
	call cphlde
	jr nz,uvv_return
	xor	a
	call put_number
uvv_return:
	pop	bc
	ret

;-----------------------------------------------------------------
; Get element in the solved bit array for current puzzle
; Output:
;	z = element
;-----------------------------------------------------------------
get_ba_solved:
	ld	hl,(d_ba_solved)
	ld	a,(d_puzzle_num)
	ld	b,a
	dec	b							; puzzle 1 is array element 0
	call get_bit_array
	ret

;-----------------------------------------------------------------
; Do necessary things when a puzzle is solved
;-----------------------------------------------------------------
puzzle_solved:
	call draw_message_rect
	ld	a,ScreenCenterY-6
	ld	hl,d_str_solved
	;set	fracDrawLFont,(iy+fontFlags)	; use small font for certain display calls
	call centertext
	;res	fracDrawLFont,(iy+fontFlags)	; use small font for certain display calls
	call get_puzzle_ptr
	ld	bc,30								; skip the puzzle data
	add	hl,bc
	ld	a,ScreenCenterY+1
	call centertext
	res	textInverse, (iy+textFlags)
	call update_screen
	call get_ba_solved
	or	(hl)
	ld	(hl),a
	call wait_for_2nd
	ret

;---------= Working RAM =---------
d_set_id:
	.db	0,0,0
d_puzzle_num:
	.db 0

d_board_set:
	.fill 30,$00
d_board_flag:
	.fill 30,$00

;---------= Data Zone =---------
d_str_detect:
	.db "TIC1DW",0
d_str_no_sets:
	.db "No puzzle sets found!"
	.db 0
d_str_saved_loading:
	.db "Loading saved puzzle..."
	.db 0
d_str_saved_alert:
	.db "Saved set not found!"
	.db 0
d_str_saved_opts:
	.db "2nd - Continue"
	.db "        MODE - Quit"
	.db 0
d_str_title:
	.db	"TIcross v1.0",0
d_str_menu_screen:
	.db "by Dan Whitman"
	.db 0
	.db "2nd - Start"
	.db 0
	.db	"ALPHA - Puzzle Set"
	.db 0
	.db	"MODE - Exit"
	.db 0
	.db	"Set: "
	.db 0
d_img_arrow_up:
	.db	5,8
	.db %00010000
	.db %00101000
	.db %01000100
	.db %10000010
	.db %11111110
d_img_arrow_down:
	.db	5,8
	.db %11111110
	.db %10000010
	.db %01000100
	.db %00101000
	.db %00010000
d_img_unsolved:
	.db %11111111,%11111110
	.db %11111111,%11111110
	.db %11111000,%00111110
	.db %11110111,%10011110
	.db %11100011,%10011110
	.db %11110111,%10011110
	.db %11111111,%00111110
	.db %11111110,%01111110
	.db %11111101,%11111110
	.db %11111101,%11111110
	.db %11111111,%11111110
	.db %11111101,%11111110
	.db %11111000,%11111110
	.db %11111101,%11111110
	.db %11111111,%11111110
d_str_solved:
	.db "Puzzle Solved!"
	.db 0
d_divider_points:
	.db	8
	.db BoardLocationX-1, BoardLocationY+1+BoardDividerLen
	.db BoardLocationX-1, BoardLocationY+1+(2*BoardDividerLen)
	.db BoardLocationX+BoardSize+1, BoardLocationY+1+BoardDividerLen
	.db BoardLocationX+BoardSize+1, BoardLocationY+1+(2*BoardDividerLen)
	.db BoardLocationX+1+BoardDividerLen, BoardLocationY-1
	.db BoardLocationX+1+(2*BoardDividerLen), BoardLocationY-1
	.db BoardLocationX+1+BoardDividerLen, BoardLocationY+BoardSize+1
	.db BoardLocationX+1+(2*BoardDividerLen), BoardLocationY+BoardSize+1
StrSavedLen equ 8
d_str_saved:
	.db " - saved"		; does not need to be null terminated, we know the length

.end
